'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _getJobKey = require('./get-job-key');

var _getJobKey2 = _interopRequireDefault(_getJobKey);

var _getQueryCost = require('./get-query-cost');

var _getQueryCost2 = _interopRequireDefault(_getQueryCost);

var _getRouteForPath2 = require('./get-route-for-path');

var _getRouteForPath3 = _interopRequireDefault(_getRouteForPath2);

var _isInTree = require('./is-in-tree');

var _isInTree2 = _interopRequireDefault(_isInTree);

var _isPluralParam = require('./is-plural-param');

var _isPluralParam2 = _interopRequireDefault(_isPluralParam);

var _partitionMutationsAndReads = require('./partition-mutations-and-reads');

var _partitionMutationsAndReads2 = _interopRequireDefault(_partitionMutationsAndReads);

var _pathsToTree = require('./paths-to-tree');

var _pathsToTree2 = _interopRequireDefault(_pathsToTree);

var _queryToPaths = require('./query-to-paths');

var _queryToPaths2 = _interopRequireDefault(_queryToPaths);

var _resolvePath = require('./resolve-path');

var _resolvePath2 = _interopRequireDefault(_resolvePath);

var _routeToParams = require('./route-to-params');

var _routeToParams2 = _interopRequireDefault(_routeToParams);

var _store = require('./store');

var _store2 = _interopRequireDefault(_store);

var _betterPromise = require('better-promise');

var _betterPromise2 = _interopRequireDefault(_betterPromise);

var _toKey = require('./to-key');

var _toKey2 = _interopRequireDefault(_toKey);

var _treeToQuery = require('./tree-to-query');

var _treeToQuery2 = _interopRequireDefault(_treeToQuery);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var runQuery = function runQuery() {
  var _ref = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {},
      _ref$query = _ref.query,
      query = _ref$query === undefined ? [] : _ref$query,
      _ref$force = _ref.force,
      force = _ref$force === undefined ? false : _ref$force,
      _ref$deltas = _ref.deltas,
      deltas = _ref$deltas === undefined ? [] : _ref$deltas,
      _ref$store = _ref.store,
      store = _ref$store === undefined ? new _store2.default() : _ref$store,
      _ref$onlyUnresolved = _ref.onlyUnresolved,
      onlyUnresolved = _ref$onlyUnresolved === undefined ? false : _ref$onlyUnresolved,
      _ref$maxQueryCost = _ref.maxQueryCost,
      maxQueryCost = _ref$maxQueryCost === undefined ? 0 : _ref$maxQueryCost,
      _ref$routes = _ref.routes,
      routes = _ref$routes === undefined ? {} : _ref$routes;

  return _betterPromise2.default.resolve().then(function () {
    if (maxQueryCost && (0, _getQueryCost2.default)(query, maxQueryCost) > maxQueryCost) {
      throw new Error('Query is too expensive');
    }

    var paths = (0, _queryToPaths2.default)(query);
    if (force !== true) {
      var forceTree = (0, _pathsToTree2.default)(force ? (0, _queryToPaths2.default)(force) : []);
      var undefPaths = [];
      for (var i = 0, l = paths.length; i < l; ++i) {
        var path = paths[i];
        var resolved = (0, _resolvePath2.default)(store.cache, path);
        if (onlyUnresolved && path === resolved) continue;

        var isUndef = store.get(resolved) === undefined;
        if (isUndef || (0, _isInTree2.default)(path, forceTree)) undefPaths.push(resolved);
      }
      paths = undefPaths;
    }

    if (!paths.length) return deltas;

    var _partitionMutationsAn = (0, _partitionMutationsAndReads2.default)(paths),
        mutations = _partitionMutationsAn.mutations,
        reads = _partitionMutationsAn.reads;

    if (mutations.length) {
      paths = mutations;
      query = [reads];
      onlyUnresolved = false;
    } else {
      query = [paths];
      force = false;
      onlyUnresolved = true;
    }

    var jobs = {};
    for (var _i = 0, _l = paths.length; _i < _l; ++_i) {
      var _path = paths[_i];

      var _getRouteForPath = (0, _getRouteForPath3.default)(routes, _path),
          route = _getRouteForPath.route,
          fn = _getRouteForPath.fn;

      var params = (0, _routeToParams2.default)(route);
      var jobKey = (0, _getJobKey2.default)(params, _path);
      var job = jobs[jobKey];
      if (!job) job = jobs[jobKey] = { fn: fn, paths: [], args: { store: store }, keys: {} };

      job.paths.push(_path);

      var _job = job,
          keys = _job.keys,
          args = _job.args;

      for (var _i2 = 0; _i2 < params.length; ++_i2) {
        var param = params[_i2];
        var arg = _path[_i2];
        if ((0, _isPluralParam2.default)(param)) {
          if (!args[_i2]) {
            args[_i2] = [];
            keys[_i2] = {};
          }
          var key = (0, _toKey2.default)(arg);
          if (keys[_i2][key]) continue;
          args[_i2].push(arg);
          keys[_i2][key] = true;
        } else args[_i2] = arg;
      }
    }

    var work = [];
    for (var _key in jobs) {
      var _jobs$_key = jobs[_key],
          fn = _jobs$_key.fn,
          _paths = _jobs$_key.paths,
          args = _jobs$_key.args;

      args.query = (0, _treeToQuery2.default)((0, _pathsToTree2.default)(_paths));
      work.push(_betterPromise2.default.resolve(args).then(fn).then(function (newDeltas) {
        store.update(newDeltas);
        deltas.push(newDeltas);
      }));
    }

    return _betterPromise2.default.all(work).then(function () {
      return runQuery({
        deltas: deltas,
        force: force,
        maxQueryCost: maxQueryCost,
        onlyUnresolved: onlyUnresolved,
        query: query,
        routes: routes,
        store: store
      });
    });
  });
};

exports.default = runQuery;
